@@ -1,7 +1,4 @@
 #!/usr/bin/env python3
-#
-# Proba wykorzystania numpy do budowy kwaternionow.
-# Trzeba porownac szybkosc dzialania obu implementacji.
 
 try:
     real_types = (int, long, float)
@@ -9,7 +6,6 @@ try:
 except NameError:   # Python 3
     real_types = (int, float)
 
-import math
 import random
 import numpy as np
 
@@ -89,33 +85,33 @@ class Quat:
     def __mul__(self, other):
         """Quaternion product."""
         other = self._normalize(other)
-        a = (self.q[0] * other.q[0] - self.q[1] * other.q[1]
+        q0 = (self.q[0] * other.q[0] - self.q[1] * other.q[1]
             - self.q[2] * other.q[2] - self.q[3] * other.q[3])
-        b = (self.q[0] * other.q[1] + self.q[1] * other.q[0]
+        q1 = (self.q[0] * other.q[1] + self.q[1] * other.q[0]
             + self.q[2] * other.q[3] - self.q[3] * other.q[2])
-        c = (self.q[0] * other.q[2] - self.q[1] * other.q[3]
+        q2 = (self.q[0] * other.q[2] - self.q[1] * other.q[3]
             + self.q[2] * other.q[0] + self.q[3] * other.q[1])
-        d = (self.q[0] * other.q[3] + self.q[1] * other.q[2]
+        q3 = (self.q[0] * other.q[3] + self.q[1] * other.q[2]
             - self.q[2] * other.q[1] + self.q[3] * other.q[0])
-        return Quat(a, b, c, d)
+        return Quat(q0, q1, q2, q3)
 
     def __rmul__(self, other):
         """Quaternion product."""
         other = self._normalize(other)
-        a = (other.q[0] * self.q[0] - other.q[1] * self.q[1]
+        q0 = (other.q[0] * self.q[0] - other.q[1] * self.q[1]
             - other.q[2] * self.q[2] - other.q[3] * self.q[3])
-        b = (other.q[0] * self.q[1] + other.q[1] * self.q[0]
+        q1 = (other.q[0] * self.q[1] + other.q[1] * self.q[0]
             + other.q[2] * self.q[3] - other.q[3] * self.q[2])
-        c = (other.q[0] * self.q[2] - other.q[1] * self.q[3]
+        q2 = (other.q[0] * self.q[2] - other.q[1] * self.q[3]
             + other.q[2] * self.q[0] + other.q[3] * self.q[1])
-        d = (other.q[0] * self.q[3] + other.q[1] * self.q[2]
+        q3 = (other.q[0] * self.q[3] + other.q[1] * self.q[2]
             - other.q[2] * self.q[1] + other.q[3] * self.q[0])
-        return Quat(a, b, c, d)
+        return Quat(q0, q1, q2, q3)
 
     def __abs__(self):
         """Return the norm of a quaternion (a scalar)."""
         powers = np.dot(self.q, self.q) # iloczyn skalarny
-        return math.sqrt(powers)
+        return np.sqrt(powers)
 
     norm = __abs__
 
@@ -141,6 +137,7 @@ class Quat:
     def __invert__(self):   # ~p, return p^{-1}
         """Reciprocal of the quaternion."""
         powers = np.dot(self.q, self.q)
+        #powers = np.sum(self.q * self.q)   # slower without numba
         return (1.0 / powers) * self.conjugate()
 
     def _pow1(self, n):
@@ -211,7 +208,7 @@ class Quat:
         vec_quat = Quat(0, vector[0], vector[1], vector[2])
         vec_quat = unit_quat * vec_quat * (~unit_quat)
         # zwracamy wektor z R^3 (wycinek)
-        return vec_quat.q[1:]   # list or numpy.array
+        return vec_quat.q[1:]   # list or tuple or numpy.array
 
     def get_rotation_matrix(self):
         """Return the rotation matrix."""
@@ -261,12 +258,12 @@ class Quat:
         The angle is in radians. The axis is a unit vector 3D."""
         if len(axis) != 3:
             raise ValueError("not a 3D vector")
-        length = math.sqrt(sum(x * x for x in axis))
+        length = np.sqrt(sum(x * x for x in axis))
         if length != 1.0:
             print("not a unit vector")
             axis = [x / length for x in axis]
-        q0 = math.cos(angle / 2.0)
-        sinus = math.sin(angle / 2.0)
+        q0 = np.cos(angle / 2.0)
+        sinus = np.sin(angle / 2.0)
         q1 = axis[0] * sinus
         q2 = axis[1] * sinus
         q3 = axis[2] * sinus
@@ -275,17 +272,17 @@ class Quat:
     @classmethod
     def from_x_rotation(cls, angle):
         """Create the unit quat for the X rotation."""
-        return cls.rot_quat([1, 0, 0], angle)
+        return cls.rot_quat((1, 0, 0), angle)
 
     @classmethod
     def from_y_rotation(cls, angle):
         """Create the unit quat for the Y rotation."""
-        return cls.rot_quat([0, 1, 0], angle)
+        return cls.rot_quat((0, 1, 0), angle)
 
     @classmethod
     def from_z_rotation(cls, angle):
         """Create the unit quat for the Z rotation."""
-        return cls.rot_quat([0, 0, 1], angle)
+        return cls.rot_quat((0, 0, 1), angle)
 
     create_from_axis_rotation = rot_quat
     create_from_x_rotation = from_x_rotation
@@ -303,19 +300,19 @@ class Quat:
     @classmethod
     def random_quat_uniax(cls):
         """Return a random rotation quat for uniaxial molecules."""
-        phi = random.uniform(0, 2*math.pi)
+        phi = random.uniform(0, 2*np.pi)
         ct = random.uniform(-1, 1)
-        theta = math.acos(ct)   # mozna tez -ct
+        theta = np.arccos(ct)   # mozna tez -ct
         quat = cls.from_eulers(phi, theta, 0)
         return quat
 
     @classmethod
     def random_quat_biax(cls):
         """Return a random rotation quat for biaxial molecules."""
-        phi = random.uniform(0, 2*math.pi)
+        phi = random.uniform(0, 2*np.pi)
         ct = random.uniform(-1, 1)
-        theta = math.acos(ct)   # mozna tez -ct
-        psi = random.uniform(0, 2*math.pi)
+        theta = np.arccos(ct)   # mozna tez -ct
+        psi = random.uniform(0, 2*np.pi)
         quat = cls.from_eulers(phi, theta, psi)
         return quat
 
@@ -336,7 +333,7 @@ class Quat:
         s3, s4 = cls.random_pair()
         S1 = s1 * s1 + s2 * s2
         S2 = s3 * s3 + s4 * s4
-        a = math.sqrt((1.0 - S1) / S2)
+        a = np.sqrt((1.0 - S1) / S2)
         return cls(s1, s2, s3 * a, s4 * a)
 
     @classmethod
@@ -345,15 +342,15 @@ class Quat:
         assert 0.0 <= ksi <= 1.0
         s1, s2 = cls.random_pair()
         S = s1 * s1 + s2 * s2
-        q1 = 2 * s1 * math.sqrt(1-S)
-        q2 = 2 * s2 * math.sqrt(1-S)
+        q1 = 2 * s1 * np.sqrt(1-S)
+        q2 = 2 * s2 * np.sqrt(1-S)
         q3 = 1 - 2 * S
         # [q1, q2, q3] is now a unit vector.
-        angle = random.uniform(-1, 1) * math.pi * 0.5 * 0.5 * ksi
-        # math.pi * 0.5 gives all rotations,
+        angle = random.uniform(-1, 1) * np.pi * 0.5 * 0.5 * ksi
+        # np.pi * 0.5 gives all rotations,
         # 0.5 is for quat cos(angle/2), sin(angle/2).
-        q0 = math.cos(angle)
-        sinus = math.sin(angle)
+        q0 = np.cos(angle)
+        sinus = np.sin(angle)
         q1 *= sinus
         q2 *= sinus
         q3 *= sinus
