@@ -11,9 +11,21 @@ import cmath
 from pyquats.quats import Quat
 #from pyquats.numpyquats import Quat
 
+def sgn(x):
+    """Return the sign of x, it is the unit quaternion for x != 0."""
+    a = abs(x)
+    if a == 0.0:
+        return Quat(0)
+    if isinstance(x, real_types):
+        return Quat(x / a)
+    elif isinstance(x, complex):
+        return Quat(x.real / a, x.imag / a)
+    else:
+        return x * Quat(1.0 / a)
+
 
 def exp(x):
-    """Return the exponential value e**x."""
+    """Return the exponential value e^x."""
     if isinstance(x, real_types):
         return Quat(math.exp(x))
     elif isinstance(x, complex):
@@ -27,12 +39,12 @@ def exp(x):
         #if v < 1e4:
         #    result = quat_imag * (1.0 - v*v/6.0)
         if v == 0.0:
-            result = quat_imag
+            return Quat(math.exp(x.q[0]))
         else:
-            result = quat_imag * (math.sin(v) / v)
-        result = result + Quat(math.cos(v))
-        result = result * math.exp(x.q[0])
-        return result
+            result = quat_imag * Quat(math.sin(v) / v)
+            result = result + Quat(math.cos(v))
+            result = result * math.exp(x.q[0])
+            return result
 
 
 def log(x):
@@ -46,7 +58,9 @@ def log(x):
 
 
 def sin(x):
-    """Return the sine of x."""
+    """Return the sine of x.
+    Note that Euler's formula is used to define sin(x).
+    """
     if isinstance(x, real_types):
         return Quat(math.sin(x))
     elif isinstance(x, complex):
@@ -54,15 +68,22 @@ def sin(x):
         return Quat(math.sin(x.real) * math.cosh(x.imag), 
                     math.cos(x.real) * math.sinh(x.imag))
     else:   # isinstance(x, Quat)
-        # sin(q)=(exp(qJ)-exp(-qJ))/(2J)
-        quat = x * Quat(0, 1)
-        result = exp(quat) - exp(-quat)
-        result = result * Quat(0, -0.5)
-        return result
+        quat_imag = Quat(0, x.q[1], x.q[2], x.q[3])
+        v = abs(quat_imag)
+        if v == 0.0:
+            return Quat(math.sin(x.q[0]))
+        else:
+            quat_imag = quat_imag * Quat(1.0 / v)   # we get a unit quat
+            quat = x * quat_imag
+            result = exp(quat) - exp(-quat)
+            result = result * quat_imag * Quat(-0.5)
+            return result
 
 
 def cos(x):
-    """Return the cosine of x."""
+    """Return the cosine of x.
+    Note that Euler's formula is used to define cos(x).
+    """
     if isinstance(x, real_types):
         return Quat(math.cos(x))
     elif isinstance(x, complex):
@@ -70,11 +91,16 @@ def cos(x):
         return Quat(math.cos(x.real) * math.cosh(x.imag), 
                    -math.sin(x.real) * math.sinh(x.imag))
     else:   # isinstance(x, Quat)
-        # cos(q)=(exp(qJ)+exp(-qJ))/2
-        quat = x * Quat(0, 1)
-        result = exp(quat) + exp(-quat)
-        result = result * Quat(0.5)
-        return result
+        quat_imag = Quat(0, x.q[1], x.q[2], x.q[3])
+        v = abs(quat_imag)
+        if v == 0.0:
+            return Quat(math.cos(x.q[0]))
+        else:
+            quat_imag = quat_imag * Quat(1.0 / v)   # we get a unit quat
+            quat = x * quat_imag
+            result = exp(quat) + exp(-quat)
+            result = result * Quat(0.5)
+            return result
 
 
 def sinh(x):
@@ -86,7 +112,7 @@ def sinh(x):
         return Quat(math.sinh(x.real) * math.cos(x.imag), 
                     math.cosh(x.real) * math.sin(x.imag))
     else:   # isinstance(x, Quat)
-        # sin(q)=(exp(q)-exp(-q))/2
+        # sinh(q)=(exp(q)-exp(-q))/2
         result = exp(x) - exp(-x)
         result = result * Quat(0.5)
         return result
@@ -101,7 +127,7 @@ def cosh(x):
         return Quat(math.cosh(x.real) * math.cos(x.imag), 
                     math.sinh(x.real) * math.sin(x.imag))
     else:   # isinstance(x, Quat)
-        # cos(q)=(exp(q)+exp(-q))/2
+        # cosh(q)=(exp(q)+exp(-q))/2
         result = exp(x) + exp(-x)
         result = result * Quat(0.5)
         return result
